//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "RGBColorInfoPanel.h"

#include <sstream>
#include <iomanip>
#include <string>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "pies"
#pragma resource "*.dfm"

const size_t TRGBColorInfoPanelForm::imageindex   = 2;
const AnsiString TRGBColorInfoPanelForm::caption  = "";
const AnsiString TRGBColorInfoPanelForm::hint     = "RGB ";

//---------------------------------------------------------------------------
__fastcall TRGBColorInfoPanelForm::TRGBColorInfoPanelForm(TComponent* Owner):
	TInfoPanelFormBase(Owner),
	_slipper(false)
{
}

//---------------------------------------------------------------------------
void TRGBColorInfoPanelForm::setdata(const Info_t &colori)
{
	RedTrackBar->Position 	= colori.red();
	GreenTrackBar->Position = colori.green();
	BlueTrackBar->Position 	= colori.blue();
}

//---------------------------------------------------------------------------
TRGBColorInfoPanelForm::Info_t TRGBColorInfoPanelForm::getdata(void) const
{
	return Info_t(RedTrackBar->Position, GreenTrackBar->Position, BlueTrackBar->Position);
}

//---------------------------------------------------------------------------
void __fastcall TRGBColorInfoPanelForm::FormCreate(TObject *Sender)
{
	_bars[RedTrackBar] 		= RedEdit;
	_bars[GreenTrackBar]	= GreenEdit;
	_bars[BlueTrackBar]		= BlueEdit;

	_fields[RedEdit]			= RedTrackBar;
	_fields[GreenEdit]		= GreenTrackBar;
	_fields[BlueEdit]			= BlueTrackBar;

	using test::RGBColorInfo::channelmax;
	
	setdata(Info_t(channelmax, channelmax, channelmax));
}

/////////////////////////////////////////////////////////////////////////////
// auxiliary
template <bool init>
struct SlipperWatcher
{
private:
	bool& _slipper;

public:
	explicit SlipperWatcher(bool &s): _slipper(s) { _slipper = init; }
	~SlipperWatcher(void) { _slipper = !_slipper; } 	
};

//---------------------------------------------------------------------------
void __fastcall TRGBColorInfoPanelForm::RedTrackBarChange(TObject *Sender)
{
	if (!_slipper)
	{
		SlipperWatcher<true> sw(_slipper);

		TTrackBar* bar 		= static_cast<TTrackBar*>(Sender);
		_bars[bar]->Text 	= AnsiString(bar->Position);

		UpdateResult();
	}
}

//---------------------------------------------------------------------------
void __fastcall TRGBColorInfoPanelForm::RedEditChange(TObject *Sender)
{
	if (!_slipper)
	{
		SlipperWatcher<true> sw(_slipper);

		TEdit* edit = static_cast<TEdit*>(Sender);

		try
		{
			_fields[edit]->Position = edit->Text.ToInt();

			UpdateResult();
		}
		catch (const EConvertError& /*e*/)
		{
			edit->Text = _fields[edit]->Position;
		}
	}
}

//---------------------------------------------------------------------------
void TRGBColorInfoPanelForm::UpdateResult(void)
{
	using std::stringstream;
	using std::hex;
	using std::showbase;
	using std::setw;
	using std::setfill;
	using std::right;

	stringstream ss;

	const Info_t c = getdata();
	ss << /*showbase*/ "0x" << setw(6) << right << setfill('0') << hex << c.color();

	RGBEdit->Text 							= ss.str().c_str();
	PreviewShape->Brush->Color 	= static_cast<TColor>(c.color());
}

//---------------------------------------------------------------------------
void TRGBColorInfoPanelForm::clear(void)
{
	setdata(Info_t());
}

//---------------------------------------------------------------------------
void TRGBColorInfoPanelForm::readonly(bool ro)
{
	RedTrackBar->Enabled 		= !ro;
	GreenTrackBar->Enabled 	= !ro;
	BlueTrackBar->Enabled 	= !ro;

	const TColor facecl     = ro? clBtnFace: clWindow;

	RedEdit->ReadOnly       = ro;
	RedEdit->Color          = facecl;

	GreenEdit->ReadOnly     = ro;
	GreenEdit->Color        = facecl;

	BlueEdit->ReadOnly      = ro;
	BlueEdit->Color         = facecl;
}


