/*
*******************************************************************************
* common.hpp
*
* Program common types declaration
*
* 2009, (c) Timoshenko D. 
*
* If some changes appear won`t forget to increase version number
*
* v. 1.0
*******************************************************************************
*/

#ifndef _COMMON_HPP_
#define _COMMON_HPP_

#include <classes.hpp>

#include <vector>

namespace test
{

/*
*******************************************************************************
* RGB Color Information Holder
*******************************************************************************
*/
struct RGBColorInfo
{
public:
	enum { channelmax = 0xff };

private:
	unsigned char _r;
	unsigned char _g;
	unsigned char _b;

public:
	explicit RGBColorInfo(unsigned char r = 0, unsigned char g = 0, unsigned char b = 0):
		_r(r), _g(g), _b(b)
	{
		// empty
	}

	explicit RGBColorInfo(unsigned int c):
		_r(0), _g(0), _b(0)
	{
		color(c);
	}

public:
	unsigned char red(void) const { return _r; }
	unsigned char green(void) const { return _g; }
	unsigned char blue(void) const { return _b; }

	void red(unsigned char r) { _r = r; }
	void green(unsigned char g) { _g = g; }
	void blue(unsigned char b) { _b = b; }

	unsigned int color(void) const { return _b << 16 | _g << 8 | _r; }

	void color(unsigned int c) { _r = c & 0xff; _g = c >> 8 & 0xff; _b = c >> 16 & 0xff; }
};

/*
*******************************************************************************
* Period information holder
*******************************************************************************
*/
struct PeriodInfo
{
private:
	TDateTime _sincedt;
	TDateTime _todt;

public:
	explicit PeriodInfo(const TDateTime& since = Now() - 1, const TDateTime& to = Now()):
		_sincedt(since),
		_todt(to)
	{
		// empty
	}

public:
	TDateTime since(void) const { return _sincedt; }
	TDateTime to(void) const { return _todt; }

	void since(const TDateTime dt) { _sincedt = dt; }
	void to(const TDateTime dt) { _todt = dt; }
};

/*
*******************************************************************************
* Days of Week Information Holder
*******************************************************************************
*/
struct WeekDayInfo
{
public:
	enum Day { monday, tuesday, wednesday, thursday, friday, saturday, sunday, dayscnt };

private:
	std::vector<bool> _weekdays;

public:
	WeekDayInfo(void): _weekdays(dayscnt, false) { /*empty*/ }

public:
	// range check
	bool day(Day d) const { return _weekdays.at(d); }

	void day(Day d, bool v) { _weekdays.at(d) = v; }

	const std::vector<bool>& week(void) const { return _weekdays; }
};

/*
*******************************************************************************
* User Information Holder
*******************************************************************************
*/
struct UserInfo 
{
private:
	AnsiString _fname;
	AnsiString _lname;
	AnsiString _username;
	AnsiString _password;

public:
	explicit UserInfo(const AnsiString &fn = "", const AnsiString &ln = "",
		const AnsiString &user = "", const AnsiString &pass = ""):
			_fname(fn),
			_lname(ln),
			_username(user),
			_password(pass)
	{
		// empty
	}

public:
 	const AnsiString& fname(void) const { return _fname; }
	const AnsiString& lname(void) const { return _lname; }
	const AnsiString& username(void) const { return _username; }
	const AnsiString& password(void) const { return _password; }

	void fname(const AnsiString& s) { _fname = s; }
	void lname(const AnsiString& s) { _lname = s; }
	void username(const AnsiString& s) { _username = s; }
	void password(const AnsiString& s) { _password = s; }
};

} // namespace test

#endif // _COMMON_HPP_
